/*
    Copyright (c) 2011-2013 Andrey Sibiryov <me@kobology.ru>
    Copyright (c) 2011-2013 Other contributors as noted in the AUTHORS file.

    This file is part of Cocaine.

    Cocaine is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    Cocaine is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include "cocaine/detail/services/logging.hpp"

#include "cocaine/api/logger.hpp"

#include "cocaine/context.hpp"
#include "cocaine/logging.hpp"
#include "cocaine/messages.hpp"

#include "cocaine/traits/enum.hpp"

using namespace cocaine::service;
using namespace std::placeholders;

logging_t::logging_t(context_t& context, io::reactor_t& reactor, const std::string& name, const Json::Value& args):
    category_type(context, reactor, name, args)
{
    auto backend = args.get("backend", "core").asString();

    if(backend != "core") {
        const auto it = context.config.loggers.find(backend);

        if(it == context.config.loggers.end()) {
            throw cocaine::error_t("the '%s' logger is not configured", backend);
        }

        m_logger = context.get<api::logger_t>(it->second.type, context.config, it->second.args);
    }

    using cocaine::logging::logger_concept_t;

    logger_concept_t* ptr = m_logger ? m_logger.get() : &context.logger();

    on<io::logging::emit>("emit", std::bind(&logger_concept_t::emit, ptr, _1, _2, _3));
    on<io::logging::verbosity>("verbosity", std::bind(&logger_concept_t::verbosity, ptr));
}

